@@ -37,9 +37,6 @@ typedef Qt3DCore::QGeometry Qt3DQGeometry;
 #include <Qt3DRender/QEffect>
 #include <Qt3DRender/QTechnique>
 #include <Qt3DRender/QGraphicsApiFilter>
-#include <Qt3DRender/QNoDepthMask>
-#include <Qt3DRender/QBlendEquation>
-#include <Qt3DRender/QBlendEquationArguments>
 #include <QMap>
 
 
@@ -138,16 +135,19 @@ QMap<QString, QString> QgsPhongMaterialSettings::toExportParameters() const
   return parameters;
 }
 
-void QgsPhongMaterialSettings::addParametersToEffect( Qt3DRender::QEffect *effect ) const
+void QgsPhongMaterialSettings::addParametersToEffect( Qt3DRender::QEffect *effect, const QgsMaterialContext &materialContext ) const
 {
+  const QColor ambient = materialContext.isSelected() ? materialContext.selectionColor().darker() : mAmbient;
+  const QColor diffuse = materialContext.isSelected() ? materialContext.selectionColor() : mDiffuse;
+
   Qt3DRender::QParameter *ambientParameter = new Qt3DRender::QParameter( QStringLiteral( "ambientColor" ),
-      QColor::fromRgbF( mAmbient.redF() * mAmbientCoefficient,
-                        mAmbient.greenF() * mAmbientCoefficient,
-                        mAmbient.blueF() * mAmbientCoefficient ) );
+      QColor::fromRgbF( ambient.redF() * mAmbientCoefficient,
+                        ambient.greenF() * mAmbientCoefficient,
+                        ambient.blueF() * mAmbientCoefficient ) );
   Qt3DRender::QParameter *diffuseParameter = new Qt3DRender::QParameter( QStringLiteral( "diffuseColor" ),
-      QColor::fromRgbF( mDiffuse.redF() * mDiffuseCoefficient,
-                        mDiffuse.greenF() * mDiffuseCoefficient,
-                        mDiffuse.blueF() * mDiffuseCoefficient ) );
+      QColor::fromRgbF( diffuse.redF() * mDiffuseCoefficient,
+                        diffuse.greenF() * mDiffuseCoefficient,
+                        diffuse.blueF() * mDiffuseCoefficient ) );
   Qt3DRender::QParameter *specularParameter = new Qt3DRender::QParameter( QStringLiteral( "specularColor" ),
       QColor::fromRgbF( mSpecular.redF() * mSpecularCoefficient,
                         mSpecular.greenF() * mSpecularCoefficient,
@@ -301,22 +301,6 @@ Qt3DRender::QMaterial *QgsPhongMaterialSettings::constantColorMaterial( const Qg
                                        mSpecular.greenF() * mSpecularCoefficient,
                                        mSpecular.blueF() * mSpecularCoefficient ) ) );
 
-  if ( mOpacity < 1.0f )
-  {
-    Qt3DRender::QNoDepthMask *noDepthMask = new Qt3DRender::QNoDepthMask( renderPass );
-
-    Qt3DRender::QBlendEquationArguments *blendState = new Qt3DRender::QBlendEquationArguments( renderPass );
-    blendState->setSourceRgb( Qt3DRender::QBlendEquationArguments::SourceAlpha );
-    blendState->setDestinationRgb( Qt3DRender::QBlendEquationArguments::OneMinusSourceAlpha );
-
-    Qt3DRender::QBlendEquation *blendEquation = new Qt3DRender::QBlendEquation( renderPass );
-    blendEquation->setBlendFunction( Qt3DRender::QBlendEquation::Add );
-
-    renderPass->addRenderState( noDepthMask );
-    renderPass->addRenderState( blendState );
-    renderPass->addRenderState( blendEquation );
-  }
-
   eff->addTechnique( technique );
   material->setEffect( eff );
 
@@ -354,22 +338,6 @@ Qt3DRender::QMaterial *QgsPhongMaterialSettings::dataDefinedMaterial() const
   eff->addParameter( new Qt3DRender::QParameter( QStringLiteral( "shininess" ),  static_cast< float >( mShininess ) ) );
   eff->addParameter( new Qt3DRender::QParameter( QStringLiteral( "opacity" ), static_cast< float >( mOpacity ) ) );
 
-  if ( mOpacity < 1.0f )
-  {
-    Qt3DRender::QNoDepthMask *noDepthMask = new Qt3DRender::QNoDepthMask( renderPass );
-
-    Qt3DRender::QBlendEquationArguments *blendState = new Qt3DRender::QBlendEquationArguments( renderPass );
-    blendState->setSourceRgb( Qt3DRender::QBlendEquationArguments::SourceAlpha );
-    blendState->setDestinationRgb( Qt3DRender::QBlendEquationArguments::OneMinusSourceAlpha );
-
-    Qt3DRender::QBlendEquation *blendEquation = new Qt3DRender::QBlendEquation( renderPass );
-    blendEquation->setBlendFunction( Qt3DRender::QBlendEquation::Add );
-
-    renderPass->addRenderState( noDepthMask );
-    renderPass->addRenderState( blendState );
-    renderPass->addRenderState( blendEquation );
-  }
-
   eff->addTechnique( technique );
   material->setEffect( eff );
 
