@@ -344,7 +344,11 @@ void QgsImageOperation::multiplyOpacity( QImage &image, const double factor, Qgs
     //decreasing opacity - we can use the faster DestinationIn composition mode
     //to reduce the alpha channel
     QColor transparentFillColor = QColor( 0, 0, 0, 255 * factor );
-    image.detach();
+    if ( image.format() == QImage::Format_Indexed8 )
+      image = image.convertToFormat( QImage::Format_ARGB32 );
+    else
+      image.detach();
+
     QPainter painter( &image );
     painter.setCompositionMode( QPainter::CompositionMode_DestinationIn );
     painter.fillRect( 0, 0, image.width(), image.height(), transparentFillColor );
