@@ -29,6 +29,7 @@
 #include "qgsoapifitemsrequest.h"
 #include "qgsoapifoptionsrequest.h"
 #include "qgsoapifqueryablesrequest.h"
+#include "qgsoapifsingleitemrequest.h"
 #include "qgswfsconstants.h"
 #include "qgswfsutils.h" // for isCompatibleType()
 
@@ -569,6 +570,7 @@ bool QgsOapifProvider::addFeatures( QgsFeatureList &flist, Flags flags )
     contentCrs = mShared->mSourceCrs.toOgcUri();
   }
   const bool hasAxisInverted = mShared->mSourceCrs.hasAxisInverted();
+  const int idFieldIdx = mShared->mFields.indexOf( "id" );
   for ( QgsFeature &f : flist )
   {
     QgsOapifCreateFeatureRequest req( uri );
@@ -579,6 +581,40 @@ bool QgsOapifProvider::addFeatures( QgsFeatureList &flist, Flags flags )
       return false;
     }
     jsonIds.append( id );
+
+    // If there's no feature["properties"]["id"] field in the JSON returned by the
+    // /items request, but there's a "id" field, it means that feature["id"]
+    // is non-numeric. Thus set the one returned by the createFeature() request
+    if ( !( flags & QgsFeatureSink::FastInsert ) &&
+         !mShared->mFoundIdInProperties && idFieldIdx >= 0 )
+    {
+      f.setAttribute( idFieldIdx, id );
+    }
+
+    // Refresh the feature content with its content from the server with a
+    // /items/{id} request.
+    if ( !( flags & QgsFeatureSink::FastInsert ) )
+    {
+      QgsOapifSingleItemRequest itemRequest( mShared->mURI.uri(), mShared->appendExtraQueryParameters( mShared->mItemsUrl + QString( QStringLiteral( "/" ) + id ) ) );
+      if ( itemRequest.request( /*synchronous=*/ true, /*forceRefresh=*/ true ) &&
+           itemRequest.errorCode() == QgsBaseNetworkRequest::NoError )
+      {
+        const QgsFeature &updatedFeature = itemRequest.feature();
+        if ( updatedFeature.isValid() )
+        {
+          int updatedFieldIdx = 0;
+          for ( const QgsField &updatedField : itemRequest.fields() )
+          {
+            const int srcFieldIdx = mShared->mFields.indexOf( updatedField.name() );
+            if ( srcFieldIdx >= 0 )
+            {
+              f.setAttribute( srcFieldIdx, updatedFeature.attribute( updatedFieldIdx ) );
+            }
+            updatedFieldIdx++;
+          }
+        }
+      }
+    }
   }
 
   QStringList::const_iterator idIt = jsonIds.constBegin();
@@ -850,9 +886,9 @@ QgsOapifSharedData *QgsOapifSharedData::clone() const
 
 static QDateTime getDateTimeValue( const QVariant &v )
 {
-  if ( v.type() == QVariant::String )
+  if ( v.userType() == QMetaType::Type::QString )
     return QDateTime::fromString( v.toString(), Qt::ISODateWithMs );
-  else if ( v.type() == QVariant::DateTime )
+  else if ( v.userType() == QMetaType::Type::QDateTime )
     return v.toDateTime();
   return QDateTime();
 }
@@ -864,9 +900,9 @@ static bool isDateTime( const QVariant &v )
 
 static QString getDateTimeValueAsString( const QVariant &v )
 {
-  if ( v.type() == QVariant::String )
+  if ( v.userType() == QMetaType::Type::QString )
     return v.toString();
-  else if ( v.type() == QVariant::DateTime )
+  else if ( v.userType() == QMetaType::Type::QDateTime )
     return v.toDateTime().toOffsetFromUtc( 0 ).toString( Qt::ISODateWithMs );
   return QString();
 }
@@ -877,7 +913,7 @@ static bool isDateTimeField( const QgsFields &fields, const QString &fieldName )
   if ( idx >= 0 )
   {
     const auto type = fields.at( idx ).type();
-    return type == QVariant::DateTime || type == QVariant::Date;
+    return type == QMetaType::Type::QDateTime || type == QMetaType::Type::QDate;
   }
   return false;
 }
@@ -966,26 +1002,26 @@ QString QgsOapifSharedData::compileExpressionNodeUsingPart1(
           if ( iter != mSimpleQueryables.end() )
           {
             if ( iter->mType == QLatin1String( "string" ) &&
-                 right->value().type() == QVariant::String )
+                 right->value().userType() == QMetaType::Type::QString )
             {
               equalityComparisons << getEncodedQueryParam( left->name(), right->value().toString() );
               removeMe = true;
             }
             else if ( ( iter->mType == QLatin1String( "integer" ) ||
                         iter->mType == QLatin1String( "number" ) ) &&
-                      right->value().type() == QVariant::Int )
+                      right->value().userType() == QMetaType::Type::Int )
             {
               equalityComparisons << getEncodedQueryParam( left->name(), QString::number( right->value().toInt() ) );
               removeMe = true;
             }
             else if ( iter->mType == QLatin1String( "number" ) &&
-                      right->value().type() == QVariant::Double )
+                      right->value().userType() == QMetaType::Type::Double )
             {
               equalityComparisons << getEncodedQueryParam( left->name(), QString::number( right->value().toDouble() ) );
               removeMe = true;
             }
             else if ( iter->mType == QLatin1String( "boolean" ) &&
-                      right->value().type() == QVariant::Bool )
+                      right->value().userType() == QMetaType::Type::Bool )
             {
               equalityComparisons << getEncodedQueryParam( left->name(), right->value().toBool() ? QLatin1String( "true" ) : QLatin1String( "false" ) );
               removeMe = true;
@@ -1386,8 +1422,8 @@ void QgsOapifFeatureDownloaderImpl::run( bool serializeFeatures, long long maxFe
           const QVariant &v = srcAttrs.value( srcIdx );
           const auto dstFieldType = dstFields.at( j ).type();
           if ( QgsVariantUtils::isNull( v ) )
-            dstFeat.setAttribute( j, QVariant( dstFieldType ) );
-          else if ( QgsWFSUtils::isCompatibleType( v.type(), dstFieldType ) )
+            dstFeat.setAttribute( j, QgsVariantUtils::createNullVariant( dstFieldType ) );
+          else if ( QgsWFSUtils::isCompatibleType( static_cast<QMetaType::Type>( v.userType() ), dstFieldType ) )
             dstFeat.setAttribute( j, v );
           else
             dstFeat.setAttribute( j, QgsVectorDataProvider::convertValue( dstFieldType, v.toString() ) );
